#include <cstdio>
#include <cstring>
#include <iconv.h>
#include <unistd.h>
#include <pqxx/pqxx>
#include <tinyptc.h>
#include "flagtrigger.h"
#include "widestring.h"
#include "fetch_current_tournament.h"
#include "fetch_list_of_active_groups.h"
#include "fetch_group.h"
#include "fetch_auxilliary_screens.h"
#include "fonts.h"
#include "groupscreen.h"
#include "top10scorescreen.h"
#include "splitscreen.h"
#include "rotatescreen.h"

Tournament active_tournament;
std::vector<SkeletonGroup> active_groups;
std::vector<GenericScreen *> screens;
GenericScreen *mainscreen = NULL;
unsigned char framebuf[800 * 600 * 4], screenbuf[800 * 600 * 4];

void init(pqxx::connection &conn)
{
	std::vector<widestring> aux_screens;
		
	if (screens.size() == 0 || mainscreen != screens[0])
		delete mainscreen;
	
	for (std::vector<GenericScreen *>::const_iterator i = screens.begin(); i != screens.end(); ++i) {
		delete *i;
	}
	screens.erase(screens.begin(), screens.end());

	conn.perform(FetchCurrentTournament(&active_tournament));
	conn.perform(FetchListOfActiveGroups(&active_groups));

	if (active_tournament.id == -1) {
		std::fprintf(stderr, "No active tournament\n");
	} else {
		std::fprintf(stderr, "Current tournament is %d\n", active_tournament.id);

		for (std::vector<SkeletonGroup>::const_iterator i = active_groups.begin(); i != active_groups.end(); ++i) {
			std::fprintf(stderr, "tourn: %u  round: %u  parallel: %u  num_machines: %u\n",
				i->tournament, i->round, i->parallel, i->num_machines);

			// memory leaks here?
			for (unsigned j = 0; j < i->num_machines; ++j) {
				RotateScreen *rs = new RotateScreen();
				screens.push_back(rs);
				rs->add_screen(new GroupScreen(conn, i->tournament, i->round, i->parallel, j, i->num_machines));
			}
		}
	}

	conn.perform(FetchAuxilliaryScreens(&aux_screens));
	for (std::vector<widestring>::const_iterator i = aux_screens.begin(); i != aux_screens.end(); ++i) {
		if (*i == widestring("top10scores")) {
			screens.push_back(new Top10ScoreScreen(conn, active_tournament.id));
			continue;
		}
		std::fprintf(stderr, "Foobarbaz?\n");
	}
	
	// hack
	screens.push_back(NULL);
	screens.push_back(NULL);
	screens.push_back(NULL);
	screens.push_back(NULL);

	if (screens[1] == NULL) {
		mainscreen = screens[0];
	} else {
		mainscreen = new SplitScreen(screens[0], screens[1], screens[2], screens[3]);
	}
}

void main_loop(pqxx::connection &conn)
{
	if (active_tournament.id == -1) {
		// No active tournament, sleep a second or so and exit
		conn.await_notification(1, 0);
		return;
	}

	if (mainscreen && mainscreen->check_invalidated()) {
		mainscreen->draw(framebuf);
		ptc_update(framebuf);
		conn.await_notification(0, 10000);
	} else {
		ptc_update(framebuf);
		conn.await_notification(0, 200000);
	}
}

int main(int argc, char **argv)
{
	ptc_open("CCBS bigscreen", 800, 600);
	
	try {
		init_freetype();
		pqxx::connection conn("dbname=ccbs host=altersex.samfundet.no user=ccbs password=GeT|>>B_");
		FlagTrigger tournament_changed(conn, "active_tournament");
		FlagTrigger rounds_changed(conn, "active_groups");
		
		// when active_tournament or active_rounds is changed, we destroy everything and start from scratch
		// (at least currently)
		for ( ;; ) {
			tournament_changed.reset_flag();
			rounds_changed.reset_flag();
			init(conn);
			do {
				main_loop(conn);
				conn.get_notifs();
			} while (!tournament_changed.get_flag() && !rounds_changed.get_flag());
			std::fprintf(stderr, "active_tournament or active_groups changed, resetting...\n");
		}
	} catch (const std::exception &e) {
		std::fprintf(stderr, "Exception: %s\n", e.what());
		exit(1);
	}
	
	return 0;
}
