#include <cstring>
#include "rotatescreen.h"

RotateScreen::RotateScreen()
	: valid(false), current_screen(0), in_fade(false)
{
	fadefrom_buf = new unsigned char[800 * 600 * 4];
}

RotateScreen::~RotateScreen()
{
	delete fadefrom_buf;
}

bool RotateScreen::check_invalidated()
{
	if (!valid)
		return true;
	if (in_fade)
		return true;
	if (needs_update())
		return true;
	
	for (unsigned i = 0; i < subscreens.size(); ++i) {
		if (subscreens[i].screen->check_invalidated())
			return true;
	}

	return false;
}

void RotateScreen::draw(unsigned char *buf)
{
	bool force = false;
	
	// if we're in a fade, complete it before doing anything else
	if (in_fade) {
		struct timeval now;
		gettimeofday(&now, NULL);

		if (!fade_found_start_time) {
			fade_found_start_time = true;
			fade_started = now;
		}
		
		double elapsed_fade = double(now.tv_sec - fade_started.tv_sec) +
			double(now.tv_usec - fade_started.tv_usec) * 1.0e-6;
		
		if (elapsed_fade > 5.5 || (!same_fade && elapsed_fade > 0.5)) {
			in_fade = false;

			// ugly hack here? :-)
			subscreens[current_screen].screen->draw(subscreens[current_screen].buf);
			
			memcpy(buf, subscreens[current_screen].buf, 800 * 600 * 4);
		} else {
			// find the fade factors
			int fr, fg, fb, fa;
			if (same_fade) {
				if (elapsed_fade < 0.5) {
					fr = fg = fb = fa = unsigned(elapsed_fade/0.5 * 256.0);
				} else {
					fr = fg = fb = fa = unsigned((elapsed_fade-0.5)/5.0 * 256.0);
				}
			} else {
				fr = fg = fb = fa = unsigned(elapsed_fade/0.5 * 256.0);
			}

			unsigned char *sptr1 = fadefrom_buf;
			unsigned char *sptr2 = subscreens[current_screen].buf;
			unsigned char *dptr = buf;

			if (same_fade && elapsed_fade >= 0.5) {
				// fade G&B to be = R
				for (unsigned i = 0; i < 800 * 600; ++i) {
					dptr[0] = sptr2[0] + (((int(sptr2[2]) - int(sptr2[0])) * fb) >> 8);
					dptr[1] = sptr2[1] + (((int(sptr2[2]) - int(sptr2[1])) * fg) >> 8);
					dptr[2] = sptr2[2];
					dptr[3] = sptr2[3];

					sptr1 += 4, sptr2 += 4, dptr += 4;
				}
			} else {
				for (unsigned i = 0; i < 800 * 600; ++i) {
					dptr[0] = sptr1[0] + (((int(sptr2[0]) - int(sptr1[0])) * fb) >> 8);
					dptr[1] = sptr1[1] + (((int(sptr2[1]) - int(sptr1[1])) * fg) >> 8);
					dptr[2] = sptr1[2] + (((int(sptr2[2]) - int(sptr1[2])) * fr) >> 8);
					dptr[3] = sptr1[3] + (((int(sptr2[3]) - int(sptr1[3])) * fa) >> 8); 

					sptr1 += 4, sptr2 += 4, dptr += 4;
				}
			}
		}
	} else {
		// determine if we want to switch screens
	
		unsigned old_current_screen = current_screen;
		
		// push any invalidated screen first (for now)
		for (unsigned i = 0; i < subscreens.size(); ++i) {
			if (subscreens[i].screen->check_invalidated()) {
				current_screen = i;
				force = true;
			}
		}

		// check if we want to go to the next screen
		if (valid && !force && needs_update()) {
			current_screen = (current_screen + 1) % subscreens.size();
			gettimeofday(&last_update, NULL);
		}

		if (current_screen != old_current_screen || subscreens[current_screen].screen->check_invalidated()) {
			// initialize a fade
			in_fade = true;
			fade_found_start_time = false;
			same_fade = (current_screen == old_current_screen);
			
			memcpy(fadefrom_buf, subscreens[old_current_screen].buf, 800 * 600 * 4);

			if (subscreens[current_screen].screen->check_invalidated())
				subscreens[current_screen].screen->draw(subscreens[current_screen].buf);
		}
	}

	if (!valid) {
		valid = true;
		gettimeofday(&last_update, NULL);
	}
}
	
// note: makes no sense if valid=false!
bool RotateScreen::needs_update()
{
	struct timeval now;
	gettimeofday(&now, NULL);

	double since = double(now.tv_sec - last_update.tv_sec) +
		double(now.tv_usec - last_update.tv_usec) * 1.0e-6;

	return (since >= 10.0);
}

void RotateScreen::add_screen(GenericScreen *screen)
{
	Subscreen ss;
	ss.buf = new unsigned char[800 * 600 * 4];
	ss.screen = screen;

	subscreens.push_back(ss);
}

